using System;
using System.Collections.Generic;
using System.Text;

namespace OhioState.Collections
{
    /// <summary>
    /// Interface for a priority-based collection or a collection with a determined 
    /// order in accessing the elements of a collection  (e.g., queue, stack, etc.).
    /// </summary>
    /// <typeparam name="T">The type of the elements in the collection.</typeparam>
    public interface IPriorityCollection<T>
    {
        /// <summary>
        /// Gets the next element in the collection.
        /// </summary>
        /// <returns>The next item in the collection.</returns>
        T GetNext();
        /// <summary>
        /// Gets the next item from the collection.
        /// </summary>
        /// <returns>The next item from the collection.</returns>
        /// <remarks>Unlike GetNext, this method does not change the underlying collection.</remarks>
        T Peek();
        /// <summary>
        /// Adds the item to the collection.
        /// </summary>
        /// <param name="item">The item to insert into the collection.</param>
        void Put(T item);
        /// <summary>
        /// The number of items currently in the collection.
        /// </summary>
        int Count { get; }
        /// <summary>
        /// Clear the buffer.
        /// </summary>
        void Clear();
        /// <summary>
        /// Checks whether the collection is full.
        /// </summary>
        /// <returns>True if the collection is full. False otherwise.</returns>
        bool IsFull();
    }

    /// <summary>
    /// A wrapper that adapts a <typeparamref name="Queue{T}"/> to a 
    /// <typeparamref name="IPriorityCollection{T}"/>.
    /// </summary>
    /// <typeparam name="T">The type of the elements in the collection.</typeparam>
    [Serializable]
    public class QueueAdaptor<T> : Queue<T>, IPriorityCollection<T>
    {
        #region Constructors
        /// <summary>
        /// Constructor specifying an initial queue size.
        /// </summary>
        /// <param name="capacity">The initial number of elements that the 
        /// <typeparamref name="QueueAdaptor{T}"/> can contain.</param>
        public QueueAdaptor(int capacity)
            : base(capacity)
        {
        }
        /// <summary>
        /// Default Constructor.
        /// </summary>
        public QueueAdaptor()
            : base()
        {
        }
        #endregion

        #region IPriorityCollection<T> Members
        /// <summary>
        /// Gets the next element in the queue.
        /// </summary>
        /// <returns>The next item in the queue.</returns>
        /// <exception cref="System.InvalidOperationException">
        /// An <paramref name="InvalidOperationException"/> is thrown if the 
        /// <typeparamref name="QueueAdaptor{T}"/> is empty.</exception>
        public T GetNext()
        {
            return this.Dequeue();
        }

        /// <summary>
        /// Adds the item to the queue.
        /// </summary>
        /// <param name="item">The item to insert into the queue.</param>
        public void Put(T item)
        {
            this.Enqueue(item);
        }

        /// <summary>
        /// Checks whether the queue is full.
        /// </summary>
        /// <returns>True if the queue is full. False otherwise.</returns>
        public bool IsFull()
        {
            return false;
        }
        #endregion
    }

    /// <summary>
    /// A wrapper that adapts a <typeparamref name="Stack{T}"/> to a 
    /// <typeparamref name="IPriorityCollection{T}"/>.
    /// </summary>
    /// <typeparam name="T">The type of the elements in the collection.</typeparam>
    [Serializable]
    public class StackAdaptor<T> : Stack<T>, IPriorityCollection<T>
    {
        #region Constructors
        /// <summary>
        /// Constructor specifying an initial stack size.
        /// </summary>
        /// <param name="capacity">The initial number of elements that the 
        /// <typeparamref name="StackAdaptor{T}"/> can contain.</param>
        public StackAdaptor(int capacity)
            : base(capacity)
        {
        }
        /// <summary>
        /// Default Constructor.
        /// </summary>
        public StackAdaptor()
            : base()
        {
        }
        #endregion

        #region IPriorityCollection<T> Members
        /// <summary>
        /// Gets the next element in the stack.
        /// </summary>
        /// <returns>The next item in the stack.</returns>
        /// <exception cref="System.InvalidOperationException">
        /// An <paramref name="InvalidOperationException"/> is thrown if the 
        /// <typeparamref name="StackAdaptor{T}"/> is empty.</exception>
        public T GetNext()
        {
            return this.Pop();
        }

        /// <summary>
        /// Adds the item to the stack.
        /// </summary>
        /// <param name="item">The item to insert into the stack.</param>
        public void Put(T item)
        {
            this.Push(item);
        }

        /// <summary>
        /// Checks whether the stack is full.
        /// </summary>
        /// <returns>True if the stack is full. False otherwise.</returns>
        public bool IsFull()
        {
            return false;
        }
        #endregion
    }

    /// <summary>
    /// A wrapper that adapts a <typeparamref name="Heap{T}"/> to a 
    /// <typeparamref name="IPriorityCollection{T}"/>.
    /// </summary>
    /// <typeparam name="T">The type of the elements in the collection.</typeparam>
    [Serializable]
    public class HeapAdaptor<T> : Heap<T>, IPriorityCollection<T>
    {
        #region Constructors
        /// <summary>
        /// Constructor specifying an inital size and an <typeparamref name="IComparer{T}"/>.
        /// </summary>
        /// <param name="capacity">The initial number of elements that the 
        /// <typeparamref name="HeapAdaptor{T}"/> can contain.</param>
        /// <param name="costComparer">The comparer to use.</param>
        public HeapAdaptor(int capacity, IComparer<T> costComparer)
            : base(capacity, costComparer)
        {
        }
        /// <summary>
        /// Constructor specifying an initial heap size.
        /// </summary>
        /// <param name="capacity">The initial number of elements that the 
        /// <typeparamref name="HeapAdaptor{T}"/> can contain.</param>
        public HeapAdaptor(int capacity)
            : base(capacity)
        {
        }
        /// <summary>
        /// Default Constructor.
        /// </summary>
        public HeapAdaptor()
            : base()
        {
        }
        #endregion

        #region IPriorityCollection<T> Members

        /// <summary>
        /// Gets the next element in the heap.
        /// </summary>
        /// <returns>The next item in the heap.</returns>
        /// <exception cref="System.InvalidOperationException">
        /// An <paramref name="InvalidOperationException"/> is thrown if the 
        /// <typeparamref name="HeapAdaptor{T}"/> is empty.</exception>
        public T GetNext()
        {
            return this.RemoveRoot();
        }

        /// <summary>
        /// Gets the next item from the heap.
        /// </summary>
        /// <returns>The next item from the heap.</returns>
        /// <remarks>Unlike GetNext, this method does not change the underlying heap.</remarks>
        /// <exception cref="System.InvalidOperationException">
        /// An <paramref name="InvalidOperationException"/> is thrown if the 
        /// <typeparamref name="HeapAdaptor{T}"/> is empty.</exception>
        public T Peek()
        {
            return this.Root;
        }

        /// <summary>
        /// Adds the item to the heap.
        /// </summary>
        /// <param name="item">The item to insert into the heap.</param>
        public void Put(T item)
        {
            this.Add(item);
        }

        /// <summary>
        /// Checks whether the heap is full.
        /// </summary>
        /// <returns>True if the heap is full. False otherwise.</returns>
        public bool IsFull()
        {
            return false;
        }
        #endregion
    }
}
